#pragma once

/*
Copyright (c) 2013-2016 Gustav Hartvigsson

Permission is hereby granted, free of charge, to any person obtaining a copy
of this software and associated documentation files (the "Software"), to deal
in the Software without restriction, including without limitation the rights
to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
copies of the Software, and to permit persons to whom the Software is
furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in
all copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
THE SOFTWARE.
 */

#include "defs.h"

S_BEGIN_DECLS

/**
 * @file
 * @addtogroup MemoryM
 * @{
 * @defgroup mmmas Mark and Sweep Memory Management.
 * @addtogroup mmmas
 * @{
 * The Mark and Sweep memory management helper functions.
 *
 * The use of "mark and sweep" is purely an experiment and should probably not
 * be used in end-user-applications.
 *
 * @note All these functions should be considered private and should not be
 * called by end users of the library.
 *
 *
 * @note This is not a garbage collector.
 */

/**
 * The initial length of the array.
 */
#define S_MM_MARK_AND_SWEEP_ARRAY_LEN 127

/**
 * Initialise the Mark and Sweep memory management system.
 */
S_HIDDEN
void
s_mm_mark_and_sweep_init ();

/**
 * Clean up the Mark and Sweep memory management system.
 */
S_HIDDEN
void
s_mm_mark_and_sweep_cleanup ();


/**
 * Add a pointer to the sweep list.
 */
S_HIDDEN
void
s_mm_mark_and_sweep_add_pointer_to_list (spointer pointer);

/**
 * Sweep the pointers. Does nothing if their are no pointer to sweep.
 */
S_HIDDEN
void
s_mm_mark_and_sweep_sweep ();

/** @} @} */

S_END_DECLS
